//
// X11 console implementation for PTC 2.0 C++ API
// Copyright (c) 1998 Christian Nentwich (brn@eleet.mcb.at)
// The PTC 2.0 C++ API is (c) 1998 Glenn Fiedler (ptc@gaffer.org)
// This source code is licensed under the GNU LGPL
//
// Please refer to the file COPYING.LIB contained in the distribution for
// licensing conditions
//


// include files
#include <X11/Xlib.h>
#include <unistd.h>
#include "Error.h"
#include "Console.h"




Console::Console()
{
    // defaults
    x11disp = 0;
}


Console::Console(const char title[],int width,int height,Format& format,
  long flags)
{
    x11disp = 0;

    // open console
    try {
      open(title,width,height,format,flags);
    }
    catch(Error e)              // have to rethrow (error in constructor) ..
    { close();

      throw e;                  // .. and so we shall
    }
}


Console::~Console()
{
    close();
}




void Console::open(const char title[],int width,int height,Format& format,
  long flags)
{
    close();

    // Check if we can open an X display
    Display *disp=XOpenDisplay(0);
    if(disp==NULL) throw Error("Cannot open X display");

    // DefaultScreen should be fine
    int screen=DefaultScreen(disp);

    // Check if nodga flag set or DGA not available
    if((flags&PTC_X11_NODGA) || !checkDGA(width,height,disp,screen))
    x11disp=new X11WindowDisplay();
#ifdef HAVE_DGA
    else
    x11disp=new X11DGADisplay();
#endif

    x11disp->open(title,width,height,format,disp,screen);
    x11disp->flags(flags);
}


void Console::open(Display *disp,int screen,Window w,Format& format,long flags)
{
  // We already have a display and a window, so we create a windowdisplay ;)
  x11disp=new X11WindowDisplay();

  x11disp->open(disp,screen,w,format);
  x11disp->flags(flags);
}

void Console::open(Display *disp,int screen,Window window,Format &format,int x,
  int y,int w,int h,long flags)
{
  x11disp=new X11WindowDisplay();
  
  x11disp->open(disp,screen,window,format,x,y,w,h);
  x11disp->flags(flags);
}

void Console::close()
{
    // close display
    if(x11disp)
    { delete x11disp;
      x11disp=0;
    }
}



void Console::copy(BaseSurface &surface)
{ throw Error("console copy is not implemented");
}


bool Console::isWindow()
{
  if(x11disp->id()==PTC_WINDOW) return true;
  else
  return false;
}

#ifdef HAVE_DGA

#include <X11/extensions/xf86dga.h>
#include <X11/extensions/xf86vmode.h>

bool Console::checkDGA(int width,int height,Display *disp,int screen)
{
  if(geteuid()) return false;

  int dga_flags;
  XF86DGAQueryDirectVideo(disp,screen,&dga_flags);

  if(!dga_flags&XF86DGADirectPresent)
  return false;

  int num_modeinfo;
  XF86VidModeModeInfo **modeinfo;
  
  XF86VidModeGetAllModeLines(disp,screen,&num_modeinfo,&modeinfo);
  if(num_modeinfo==0) return false;

  bool found=false;
  for(int i=0;i<num_modeinfo;i++)
    {
      if(modeinfo[i]->hdisplay==width && modeinfo[i]->vdisplay==height)
      { found=true;
        break;
      }
    }

  if(!found) return false;

  XFree(modeinfo);

  return true;
}

#else

bool Console::checkDGA(int width,int height,Display *disp,int screen)
{ return false;
}

#endif
